@@ -169,15 +169,25 @@ set_hdr_data_slots (fr_header *hdr)
 
   nSlots = hdr->frame_slots - hdr->padding;
 
+#if 0
   if (hdr->version == MPEG_VERSION_1) {
-    nSlots = MPEG1_slot_table[hdr->srate_idx][hdr->bitrate_idx];
+    g_print ("Calced %d main slots, table says %d\n", nSlots, 
+       MPEG1_slot_table[hdr->srate_idx][hdr->bitrate_idx]);
+  } else {
+    g_print ("Calced %d main slots, table says %d\n", nSlots, 
+       MPEG2_LSF_slot_table[hdr->srate_idx][hdr->bitrate_idx]);
+  }
+#endif
+
+  if (hdr->version == MPEG_VERSION_1) {
+    // nSlots = MPEG1_slot_table[hdr->srate_idx][hdr->bitrate_idx];
 
     if (hdr->channels == 1)
       hdr->side_info_slots = MPEG1_MONO_SI_SLOTS;
     else
       hdr->side_info_slots = MPEG1_STEREO_SI_SLOTS;
   } else {
-    nSlots = MPEG2_LSF_slot_table[hdr->srate_idx][hdr->bitrate_idx];
+    // nSlots = MPEG2_LSF_slot_table[hdr->srate_idx][hdr->bitrate_idx];
 
     if (hdr->channels == 1)
       hdr->side_info_slots = MPEG2_LSF_MONO_SI_SLOTS;
@@ -193,6 +203,9 @@ set_hdr_data_slots (fr_header *hdr)
   if (hdr->error_protection)
     nSlots -= 2;
 
+  if (nSlots < 0)
+    return FALSE;
+
   hdr->main_slots = nSlots;
 
   return TRUE;
@@ -520,7 +533,7 @@ II_dequant_and_scale_sample (
         for (gr = 0; gr < 3; gr++) {
           /* locate MSB in the sample */
           x = 0;
-          while ((1L << x) < (*alloc)[sb][allocation].steps)
+          while ((1UL << x) < (*alloc)[sb][allocation].steps)
             x++;
 
           /* MSB inversion */
