@@ -75,6 +75,7 @@ flump3dec_get_type (void)
       sizeof (FluMp3Dec),
       0,
       (GInstanceInitFunc) flump3dec_init,
+      NULL
     };
     flump3dec_type = g_type_register_static (GST_TYPE_ELEMENT,
         "FluMp3Dec", &flump3dec_info, 0);
@@ -102,12 +103,13 @@ static void
 flump3dec_base_init (FluMp3DecClass *klass)
 {
   /* FIXME: Which email address to put? */
-  static GstElementDetails details = {
-    "Fluendo MP3 Decoder",
-    "Audio/Decoder",
-    "Decodes MPEG-1 Layer 1, 2 and 3 streams to raw audio frames",
-    "Fluendo Support <support@fluendo.com>"
-  };
+  static GstElementDetails details = 
+    GST_ELEMENT_DETAILS (
+      "Fluendo MP3 Decoder",
+      "Audio/Decoder",
+      "Decodes MPEG-1 Layer 1, 2 and 3 streams to raw audio frames",
+      "Fluendo Support <support@fluendo.com>"
+    );
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
   gst_element_class_add_pad_template (element_class,
@@ -123,15 +125,13 @@ flump3dec_init (FluMp3Dec *flump3dec)
 {
   /* Create and add pads */
   flump3dec->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-          (&flump3dec_sink_factory), "sink");
+      gst_pad_new_from_static_template (&flump3dec_sink_factory, "sink");
   gst_pad_set_event_function (flump3dec->sinkpad, flump3dec_sink_event);
   gst_pad_set_chain_function (flump3dec->sinkpad, flump3dec_sink_chain);
   gst_element_add_pad (GST_ELEMENT (flump3dec), flump3dec->sinkpad);
 
   flump3dec->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get
-          (&flump3dec_src_factory), "src");
+      gst_pad_new_from_static_template (&flump3dec_src_factory, "src");
   gst_pad_set_query_type_function (flump3dec->srcpad, flump3dec_get_query_types);
   gst_pad_set_query_function (flump3dec->srcpad, flump3dec_src_query);
   gst_pad_set_event_function (flump3dec->srcpad, flump3dec_src_event);
@@ -243,7 +243,7 @@ flump3dec_check_for_xing (FluMp3Dec *flump3dec, const fr_header *mp3hdr)
   read_id = bs_getbits (flump3dec->bs, 32);
   if (read_id == xing_id || read_id == info_id) {
     guint32 xing_flags;
-    gint bytes_needed = 0;
+    guint bytes_needed = 0;
 
     /* Read 4 base bytes of flags, big-endian */
     xing_flags = bs_getbits (flump3dec->bs, 32);
@@ -312,6 +312,8 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
   FluMp3Dec        *flump3dec = FLUMP3DEC (GST_PAD_PARENT (pad));
   GstClockTime    dec_ts;
 
+  GST_DEBUG_OBJECT (flump3dec, "draining, more: %d", more_data);
+
   mp3tl_set_eos (flump3dec->dec, more_data);
   
   while (bs_bits_avail (flump3dec->bs) > 0)
@@ -419,7 +421,7 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
         break;
       } 
       else if (result == MP3TL_ERR_BAD_FRAME) {
-        /* Update time, and output previous frame if we have one */
+        /* Update time, and repeat the previous frame if we have one */
         flump3dec->bad = TRUE;
 	if (flump3dec->pending_frame != NULL) {
 	  GST_DEBUG_OBJECT (flump3dec, "Bad frame - using previous frame");
@@ -463,20 +465,28 @@ flump3dec_drain_avail (GstPad *pad, gboolean more_data)
                                     mp3hdr->frame_samples, mp3hdr->sample_rate);
     flump3dec->next_ts += GST_BUFFER_DURATION (out_buf);
 
-    GST_DEBUG_OBJECT (flump3dec, "Pushing buffer size %u, ts % " 
-        G_GINT64_FORMAT, GST_BUFFER_SIZE (out_buf), 
-        GST_BUFFER_TIMESTAMP (out_buf));
+    GST_DEBUG_OBJECT (flump3dec, "Have new buffer, size %u, ts %" 
+        GST_TIME_FORMAT, GST_BUFFER_SIZE (out_buf), 
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (out_buf)));
 
     if (flump3dec->pending_frame == NULL) {
+      GST_DEBUG_OBJECT (flump3dec, "Storing as pending frame"); 
       flump3dec->pending_frame = out_buf;
     }
     else {
-      ret = gst_pad_push (flump3dec->srcpad, out_buf);
+      /* push previous frame, queue current frame. */
+      GST_DEBUG_OBJECT (flump3dec, "Pushing previous frame, ts %"GST_TIME_FORMAT,
+	      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (flump3dec->pending_frame))); 
+      ret = gst_pad_push (flump3dec->srcpad, flump3dec->pending_frame);
+      flump3dec->pending_frame = out_buf;
       if (ret != GST_FLOW_OK) 
         goto error_out;
     }
   }
+
+  /* Might need to flush out the pending decoded frame */
   if (!more_data && (flump3dec->pending_frame != NULL)) {
+    GST_DEBUG_OBJECT (flump3dec, "Pushing pending frame"); 
     ret = gst_pad_push (flump3dec->srcpad, flump3dec->pending_frame);
     flump3dec->pending_frame = NULL;
     if (ret != GST_FLOW_OK) 
@@ -529,6 +539,7 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
       GST_DEBUG_OBJECT (flump3dec, 
           "new segment, format=%d, base=%lld, start = %lld, end=%lld", format, base,
 	  start, end);
+
       if (format == GST_FORMAT_BYTES) {
         gint64 disc_start, disc_end, disc_base;
 
@@ -548,11 +559,13 @@ flump3dec_sink_event (GstPad *pad, GstEvent *event)
       } 
       else if (format == GST_FORMAT_TIME)
       {
+        GST_DEBUG_OBJECT (flump3dec, "Got segment in time format");
         flump3dec->next_ts = start;
       }
 
       if (!converted && format != GST_FORMAT_TIME) {
         gst_event_unref (event);
+        GST_DEBUG_OBJECT (flump3dec, "creating new time segment");
         event = gst_event_new_new_segment (FALSE, rate, GST_FORMAT_TIME,
                     0, GST_CLOCK_TIME_NONE, 0);
         flump3dec->next_ts = 0;
@@ -592,6 +605,8 @@ flump3dec_change_state (GstElement *element, GstStateChange transition)
       dec->bitrate_sum = 0;
       dec->frame_count = 0;
       dec->xing_flags  = 0;
+      dec->channels    = 0;
+      dec->rate        = 0;
       break;
     default:
       break;
@@ -611,7 +626,7 @@ flump3dec_change_state (GstElement *element, GstStateChange transition)
 }
 
 static const GstQueryType *
-flump3dec_get_query_types (GstPad *pad)
+flump3dec_get_query_types (GstPad *pad __attribute__ ((unused)))
 {
   static const GstQueryType query_types[] = {
     GST_QUERY_POSITION,
