@@ -10,18 +10,16 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <gst/gst.h>
 #include "mp3tl-priv.h"
 #include "decode.h"
 #include "huffman.h"
-#include "table-dewindow.h"
+#include "mp3-cos-tables.h"
 
 GST_DEBUG_CATEGORY_EXTERN (flump3debug);
 #define GST_CAT_DEFAULT flump3debug
 
-#define OPT_IMDCT
-#define OPT_COS
-
 /************************* Layer III routines **********************/
 
 gboolean
@@ -114,7 +112,7 @@ III_get_side_info (Bit_stream_struc *bs, III_side_info_t *si,
           /* Set region_count parameters since they are 
            * implicit in this case. */
           if (gi->block_type == 0) {
-            g_warning ("Side info bad: block_type == 0 in split block.\n");
+            GST_WARNING ("Side info bad: block_type == 0 in split block.\n");
             return FALSE;
           } else if (gi->block_type == 2
               && gi->mixed_block_flag == 0) {
@@ -141,33 +139,75 @@ III_get_side_info (Bit_stream_struc *bs, III_side_info_t *si,
   return TRUE;
 }
 
-const gint slen[2][16] = { {0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4},
-{0, 1, 2, 3, 0, 1, 2, 3, 1, 2, 3, 1, 2, 3, 2, 3}
+const gint slen[2][16] = 
+{ 
+  {0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4},
+  {0, 1, 2, 3, 0, 1, 2, 3, 1, 2, 3, 1, 2, 3, 2, 3}
 };
 
 struct
 {
   gint l[23];
   gint s[14];
-} const sfBandIndex[6] = { { {
-0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
-          284, 336, 396, 464, 522, 576}, {
-0, 4, 8, 12, 18, 24, 32, 42, 56, 74, 100, 132, 174, 192}}, { {
-0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 114, 136, 162, 194, 232, 278,
-        330, 394, 464, 540, 576}, {
-0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 136, 180, 192}}, { {
-0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 284,
-        336, 396, 464, 522, 576}, {
-0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192}}, { {
-0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 52, 62, 74, 90, 110, 134, 162, 196,
-        238, 288, 342, 418, 576}, {
-0, 4, 8, 12, 16, 22, 30, 40, 52, 66, 84, 106, 136, 192}}, { {
-0, 4, 8, 12, 16, 20, 24, 30, 36, 42, 50, 60, 72, 88, 106, 128, 156, 190,
-        230, 276, 330, 384, 576}, {
-0, 4, 8, 12, 16, 22, 28, 38, 50, 64, 80, 100, 126, 192}}, { {
-0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 54, 66, 82, 102, 126, 156, 194, 240,
-        296, 364, 448, 550, 576}, {
-0, 4, 8, 12, 16, 22, 30, 42, 58, 78, 104, 138, 180, 192}}};
+} const sfBandIndex[] = 
+{ 
+  /* MPEG-1 */
+  { 
+    /* 44.1 khz */
+    { 0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 52, 62, 74, 90, 110, 134, 162, 196,
+      238, 288, 342, 418, 576 }, 
+    { 0, 4, 8, 12, 16, 22, 30, 40, 52, 66, 84, 106, 136, 192 }
+  }, { 
+    /* 48khz */
+    { 0, 4, 8, 12, 16, 20, 24, 30, 36, 42, 50, 60, 72, 88, 106, 128, 156, 190,
+      230, 276, 330, 384, 576 }, 
+    { 0, 4, 8, 12, 16, 22, 28, 38, 50, 64, 80, 100, 126, 192 }
+  }, { 
+    /* 32khz */
+    { 0, 4, 8, 12, 16, 20, 24, 30, 36, 44, 54, 66, 82, 102, 126, 156, 194, 240,
+      296, 364, 448, 550, 576 }, 
+    { 0, 4, 8, 12, 16, 22, 30, 42, 58, 78, 104, 138, 180, 192 }
+  },
+  /* MPEG-2 */
+  { 
+    /* 22.05 khz */
+    { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238,
+      284, 336, 396, 464, 522, 576 }, 
+    { 0, 4, 8, 12, 18, 24, 32, 42, 56, 74, 100, 132, 174, 192 }
+  }, { 
+    /* 24khz */
+    { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 114, 136, 162, 194, 232, 
+      278, 330, 394, 464, 540, 576 }, 
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 136, 180, 192 }
+  }, 
+  { 
+    /* 16 khz */
+    { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 
+      284, 336, 396, 464, 522, 576 }, 
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+  }, 
+  /* MPEG-2.5 */
+  {
+    /* 11025 */
+    { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 
+      284, 336, 396, 464, 522, 576 }, 
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+  }, 
+  { 
+    /* 12khz */
+    { 0, 6, 12, 18, 24, 30, 36, 44, 54, 66, 80, 96, 116, 140, 168, 200, 238, 
+      284, 336, 396, 464, 522, 576 }, 
+    { 0, 4, 8, 12, 18, 26, 36, 48, 62, 80, 104, 134, 174, 192 }
+  }, 
+  { 
+    /* 8khz */
+    { 0, 12, 24, 36, 48, 60, 72, 88, 108, 132, 160, 192, 232, 280, 336, 400,
+      476, 566, 568, 570, 572, 574, 576 },
+    { 0, 8, 16, 24, 36, 52, 72, 96, 124, 160, 162, 164, 166, 192 }
+  }
+};
+/* Offset into the sfBand table for each MPEG version */
+guint sfb_offset[] = { 6, 0 /* invalid */, 3, 0 };
 
 void III_get_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
      int gr, int ch, mp3tl *tl)
@@ -211,7 +251,6 @@ void III_get_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
   } else {
     gint i;
     const gint l_sfbtable[5] = { 0, 6, 11, 16, 21 };
-
     /* LONG types 0,1,3 */
     if (gr == 0) {
       for (sfb = 0; sfb < 11; sfb++) {
@@ -244,22 +283,22 @@ void III_get_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
 /****************** new MPEG2 stuf  ***********/
 
 static const guint nr_of_sfb_block[6][3][4] =
-    { {{6, 5, 5, 5}, {9, 9, 9, 9}, {6, 9, 9, 9}},
-{{6, 5, 7, 3}, {9, 9, 12, 6}, {6, 9, 12, 6}},
-{{11, 10, 0, 0}, {18, 18, 0, 0}, {15, 18, 0, 0}},
-{{7, 7, 7, 0}, {12, 12, 12, 0}, {6, 15, 12, 0}},
-{{6, 6, 6, 3}, {12, 9, 9, 6}, {6, 12, 9, 6}},
-{{8, 8, 5, 0}, {15, 12, 9, 0}, {6, 18, 9, 0}}
+    { 
+      { {6,  5,  5, 5}, {9,  9,  9,  9}, {6,  9,  9,  9} },
+      { {6,  5,  7, 3}, {9,  9,  12, 6}, {6,  9,  12, 6} },
+      { {11, 10, 0, 0}, {18, 18, 0,  0}, {15, 18, 0,  0} },
+      { {7,  7,  7, 0}, {12, 12, 12, 0}, {6,  15, 12, 0} },
+      { {6,  6,  6, 3}, {12, 9,  9,  6}, {6,  12, 9,  6} },
+      { {8,  8,  5, 0}, {15, 12, 9,  0}, {6,  18, 9,  0} }
 };
 
 void
-III_get_LSF_scale_data (III_scalefac_t *scalefac, III_side_info_t *si,
+III_get_LSF_scale_data (guint *scalefac_buffer, III_side_info_t *si,
     gint gr, gint ch, mp3tl *tl)
 {
   short i, j, k;
   short blocktypenumber;
   short blocknumber = -1;
-  guint *scalefac_buffer;
 
   gr_info_t *gr_info = &(si->gr[gr][ch]);
   guint scalefac_comp, int_scalefac_comp, new_slen[4];
@@ -268,7 +307,6 @@ III_get_LSF_scale_data (III_scalefac_t *scalefac, III_side_info_t *si,
   fr_header *hdr = &tl->fr_ps.header;
 
   scalefac_comp = gr_info->scalefac_compress;
-  scalefac_buffer = tl->c_impl.scalefac_buffer;
 
   blocktypenumber = 0;
   if ((gr_info->block_type == 2) && (gr_info->mixed_block_flag == 0))
@@ -366,9 +404,8 @@ void III_get_LSF_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
   gr_info_t *gr_info = &(si->gr[gr][ch]);
   guint *scalefac_buffer;
 
-  III_get_LSF_scale_data (scalefac, si, gr, ch, tl);
-
   scalefac_buffer = tl->c_impl.scalefac_buffer;
+  III_get_LSF_scale_data (scalefac_buffer, si, gr, ch, tl);
 
   if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
     if (gr_info->mixed_block_flag) {    /* MIXED *//* NEW - ag 11/25 */
@@ -383,7 +420,6 @@ void III_get_LSF_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
         }
       for (sfb = 12, window = 0; window < 3; window++)
         (*scalefac)[ch].s[window][sfb] = 0;
-
     } else {                    /* SHORT */
       for (sfb = 0; sfb < 12; sfb++)
         for (window = 0; window < 3; window++) {
@@ -398,30 +434,28 @@ void III_get_LSF_scale_factors (III_scalefac_t *scalefac, III_side_info_t *si,
       (*scalefac)[ch].l[sfb] = scalefac_buffer[k];
       k++;
     }
-    (*scalefac)[ch].l[22] = 0;
+    (*scalefac)[ch].l[21] = 0;
   }
 }
 
 gboolean
-III_huffman_decode (glong is[SBLIMIT][SSLIMIT], III_side_info_t *si,
+III_huffman_decode (gint is[SBLIMIT][SSLIMIT], III_side_info_t *si,
     gint ch, gint gr, gint part2_start, mp3tl *tl)
 {
-  int i, x, y;
+  guint i;
+  int x, y;
   int v, w;
   gint h; /* Index of the huffman table to use */
-  int region1Start;
-  int region2Start;
+  guint region1Start;
+  guint region2Start;
   int sfreq;
-  int currentBit, grBits;
+  guint currentBit, grBits;
   gr_info_t *gi = &(si->gr[gr][ch]);
   huffdec_bitbuf *bb = &tl->c_impl.bb;
   frame_params *fr_ps = &tl->fr_ps;
 
-  /* Calculate index. Offset by 3 for mpeg2 */
-  if (fr_ps->header.version == MPEG_VERSION_1)
-    sfreq = fr_ps->header.srate_idx + 3;
-  else
-    sfreq = fr_ps->header.srate_idx;
+  /* Calculate index. */
+  sfreq = sfb_offset[fr_ps->header.version] + fr_ps->header.srate_idx;
     
   /* Find region boundary for short block case. */
   if ((gi->window_switching_flag) &&
@@ -438,7 +472,6 @@ III_huffman_decode (glong is[SBLIMIT][SSLIMIT], III_side_info_t *si,
   }
 
   grBits = part2_start + gi->part2_3_length;
-  currentBit = h_sstell (bb);
 
   /* Read bigvalues area. */
   /* i < SSLIMIT * SBLIMIT => gi->big_values < SSLIMIT * SBLIMIT/2 */
@@ -456,13 +489,10 @@ III_huffman_decode (glong is[SBLIMIT][SSLIMIT], III_side_info_t *si,
     is[(i + 1) / SSLIMIT][(i + 1) % SSLIMIT] = y;
   }
 
-  grBits = part2_start + gi->part2_3_length;
-  currentBit = h_sstell (bb);
-
   /* Read count1 area. */
   h = gi->count1table_select + 32;
   while ((h_sstell (bb) < part2_start + gi->part2_3_length) &&
-      (i < SSLIMIT * SBLIMIT)) {
+      (i + 3) < (SBLIMIT * SSLIMIT)) {
     if (!huffman_decoder (bb, h, &x, &y, &v, &w))
       return FALSE;
 
@@ -474,7 +504,6 @@ III_huffman_decode (glong is[SBLIMIT][SSLIMIT], III_side_info_t *si,
   }
 
   grBits = part2_start + gi->part2_3_length;
-  currentBit = h_sstell (bb);
 
   if (h_sstell (bb) > grBits) {
     i -= 4;
@@ -486,6 +515,8 @@ III_huffman_decode (glong is[SBLIMIT][SSLIMIT], III_side_info_t *si,
   if (currentBit < grBits)
     h_getbits (bb, grBits - currentBit);
 
+  g_assert (i <= SSLIMIT * SBLIMIT);
+
   /* Zero out rest. */
   for (; i < SSLIMIT * SBLIMIT; i++)
     is[i / SSLIMIT][i % SSLIMIT] = 0;
@@ -499,7 +530,7 @@ const gint pretab[22] =
 
 void
 III_dequantize_sample (
-     glong is[SBLIMIT][SSLIMIT],
+     gint is[SBLIMIT][SSLIMIT],
      gfloat xr[SBLIMIT][SSLIMIT],
      III_scalefac_t *scalefac,
      gr_info_t *gr_info,
@@ -512,35 +543,34 @@ III_dequantize_sample (
   int next_cb_boundary;
   int cb_begin = 0;
   int cb_width = 0;
-  int sign;
+  gint tmp;
   gfloat global_gain;
   gfloat is_short_blk;
 
-  /* band table index. offset by 3 for MPEG-2 */
-  if (fr_ps->header.version == MPEG_VERSION_1)
-    sfreq = fr_ps->header.srate_idx + 3;
-  else
-    sfreq = fr_ps->header.srate_idx;
+  /* Calculate index. */
+  sfreq = sfb_offset[fr_ps->header.version] + fr_ps->header.srate_idx;
 
   /* choose correct scalefactor band per block type, initalize boundary */
-  if (gr_info->window_switching_flag && (gr_info->block_type == 2))
-    if (gr_info->mixed_block_flag)
+  if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
+    if (gr_info->mixed_block_flag) {
       next_cb_boundary = sfBandIndex[sfreq].l[1];       /* LONG blocks: 0,1,3 */
-    else {
+    } else {
       next_cb_boundary = sfBandIndex[sfreq].s[1] * 3;   /* pure SHORT block */
       cb_width = sfBandIndex[sfreq].s[1];
       cb_begin = 0;
-  } else
+    }
+  } else {
     next_cb_boundary = sfBandIndex[sfreq].l[1]; /* LONG blocks: 0,1,3 */
+  }
 
   global_gain = pow (2.0, (0.25 * (gr_info->global_gain - 210.0)));
   /* apply formula per block type */
   for (sb = 0; sb < SBLIMIT; sb++) {
     gint sb_off = sb * 18;
-    is_short_blk =  (gr_info->window_switching_flag && (
+    is_short_blk =  gr_info->window_switching_flag && (
               ((gr_info->block_type == 2) && (gr_info->mixed_block_flag == 0))
               || ((gr_info->block_type == 2) && gr_info->mixed_block_flag
-                  && (sb >= 2))));
+                  && (sb >= 2)));
 
     for (ss = 0; ss < SSLIMIT; ss++) {
       if (sb_off + ss == next_cb_boundary) { /* Adjust critical band boundary */
@@ -552,7 +582,7 @@ III_dequantize_sample (
               cb_width = sfBandIndex[sfreq].s[cb + 1] -
                   sfBandIndex[sfreq].s[cb];
               cb_begin = sfBandIndex[sfreq].s[cb] * 3;
-            } else if (((sb * 18) + ss) < sfBandIndex[sfreq].l[8])
+            } else if ((sb_off + ss) < sfBandIndex[sfreq].l[8])
               next_cb_boundary = sfBandIndex[sfreq].l[(++cb) + 1];
             else {
               next_cb_boundary = sfBandIndex[sfreq].s[(++cb) + 1] * 3;
@@ -575,7 +605,7 @@ III_dequantize_sample (
       /* Do long/short dependent scaling operations. */
       if (is_short_blk) {
         xr[sb][ss] *= pow (2.0, 0.25 * -8.0 *
-            gr_info->subblock_gain[(((sb * 18) + ss) - cb_begin) / cb_width]);
+            gr_info->subblock_gain[((sb_off + ss) - cb_begin) / cb_width]);
         xr[sb][ss] *= pow (2.0, 0.25 * -2.0 * (1.0 + gr_info->scalefac_scale)
             * (*scalefac)[ch].s[(sb_off + ss - cb_begin) / cb_width][cb]);
       } else {
@@ -585,10 +615,13 @@ III_dequantize_sample (
       }
 
       /* Scale quantized value. */
-      sign = (is[sb][ss] < 0) ? 1 : 0;
-      xr[sb][ss] *= powf (abs (is[sb][ss]), (4.0 / 3.0));
-      if (sign)
-        xr[sb][ss] = -xr[sb][ss];
+      tmp = is[sb][ss];
+      if (tmp >= 0) {
+        xr[sb][ss] *= powf (tmp, (4.0 / 3.0));
+      }
+      else {
+        xr[sb][ss] *= -1.0 * powf (-tmp, (4.0 / 3.0));
+      }
     }
   }
 }
@@ -601,17 +634,14 @@ III_reorder (gfloat xr[SBLIMIT][SSLIMIT], gfloat ro[SBLIMIT][SSLIMIT],
   int sfb, sfb_start, sfb_lines;
   int sb, ss, window, freq, src_line, des_line;
 
-  /* band table index. offset by 3 for MPEG-2 */
-  if (fr_ps->header.version == MPEG_VERSION_1)
-    sfreq = fr_ps->header.srate_idx + 3;
-  else
-    sfreq = fr_ps->header.srate_idx;
-
-  for (sb = 0; sb < SBLIMIT; sb++)
-    for (ss = 0; ss < SSLIMIT; ss++)
-      ro[sb][ss] = 0;
+  /* Calculate index. */
+  sfreq = sfb_offset[fr_ps->header.version] + fr_ps->header.srate_idx;
 
   if (gr_info->window_switching_flag && (gr_info->block_type == 2)) {
+    for (sb = 0; sb < SBLIMIT; sb++)
+      for (ss = 0; ss < SSLIMIT; ss++)
+        ro[sb][ss] = 0;
+
     if (gr_info->mixed_block_flag) {
       /* NO REORDER FOR LOW 2 SUBBANDS */
       for (sb = 0; sb < 2; sb++)
@@ -689,11 +719,8 @@ III_stereo (gfloat xr[2][SBLIMIT][SSLIMIT], gfloat lr[2][SBLIMIT][SSLIMIT],
     io = 0.840896415256;
   }
 
-  /* band table index. offset by 3 for MPEG-2 */
-  if (fr_ps->header.version == MPEG_VERSION_1)
-    sfreq = fr_ps->header.srate_idx + 3;
-  else
-    sfreq = fr_ps->header.srate_idx;
+  /* Calculate index. */
+  sfreq = sfb_offset[fr_ps->header.version] + fr_ps->header.srate_idx;
 
   /* intialization */
   for (i = 0; i < SBLIMIT * SSLIMIT; i++)
@@ -935,8 +962,7 @@ static const gfloat Ci[8] =
 void
 III_antialias (gfloat xr[SBLIMIT][SSLIMIT],
      gfloat hybridIn[SBLIMIT][SSLIMIT],
-     gr_info_t *gr_info,
-     frame_params *fr_ps)
+     gr_info_t *gr_info)
 {
   /* Static shared computed constants */
   static int init = 0;
@@ -988,94 +1014,17 @@ III_antialias (gfloat xr[SBLIMIT][SSLIMIT],
 static inline void
 imdct_9pt (gfloat invec[9], gfloat outvec[9]);
 
-/* pre-calculated cos() in steps of PI/72 and PI/24. The Octave
- * formula to generate each table is given in the comment above
- */
-#ifdef OPT_COS
-
-/* 0.5 ./ cos ((1:12) .* pi/24) */
-static const gdouble cos24_table[] = {
-  5.043144802900764167574720886477734893560409545898437500000000e-01,
-  5.176380902050414789528076653368771076202392578125000000000000e-01,
-  5.411961001461970122150546558259520679712295532226562500000000e-01,
-  5.773502691896257310588680411456152796745300292968750000000000e-01,
-  6.302362070051322762154200063378084450960159301757812500000000e-01,
-  7.071067811865474617150084668537601828575134277343750000000000e-01,
-  8.213398158522907666068135768000502139329910278320312500000000e-01,
-  9.999999999999997779553950749686919152736663818359375000000000e-01,
-  1.306562964876376353728915091778617352247238159179687500000000e+00,
-  1.931851652578136846472034449107013642787933349609375000000000e+00,
-  3.830648787770190910606515899416990578174591064453125000000000e+00,
-  8.165889364191922000000000000000000000000000000000000000000000e+15
-};
-
-/* cos ((0:8) .* pi/18) */
-static const gdouble cos18_table[] = {
-  1.000000000000000000000000000000000000000000000000000000000000e+00,
-  9.848077530122080203156542665965389460325241088867187500000000e-01,
-  9.396926207859084279050421173451468348503112792968750000000000e-01,
-  8.660254037844387076106045242340769618749618530273437500000000e-01,
-  7.660444431189780134516809084743726998567581176757812500000000e-01,
-  6.427876096865393629187224178167525678873062133789062500000000e-01,
-  5.000000000000001110223024625156540423631668090820312500000000e-01,
-  3.420201433256688239303855425532674416899681091308593750000000e-01,
-  1.736481776669304144533612088707741349935531616210937500000000e-01
-};
-
-/* 0.5 ./ cos ((1:35) .* pi/72)) */
-static const gdouble icos72_table[] = {
-  5.004763425816599609063928255636710673570632934570312500000000e-01,
-  5.019099187716736798492433990759309381246566772460937500000000e-01,
-  5.043144802900764167574720886477734893560409545898437500000000e-01,
-  5.077133059428725614381505693017970770597457885742187500000000e-01,
-  5.121397571572545714957414020318537950515747070312500000000000e-01,
-  5.176380902050414789528076653368771076202392578125000000000000e-01,
-  5.242645625704053236049162478593643754720687866210937500000000e-01,
-  5.320888862379560269033618169487453997135162353515625000000000e-01,
-  5.411961001461970122150546558259520679712295532226562500000000e-01,
-  5.516889594812458552652856269560288637876510620117187500000000e-01,
-  5.636909734331712051869089918909594416618347167968750000000000e-01,
-  5.773502691896257310588680411456152796745300292968750000000000e-01,
-  5.928445237170802961657045671017840504646301269531250000000000e-01,
-  6.103872943807280293526673631276935338973999023437500000000000e-01,
-  6.302362070051321651931175438221544027328491210937500000000000e-01,
-  6.527036446661392821155800447741057723760604858398437500000000e-01,
-  6.781708524546284921896699415810871869325637817382812500000000e-01,
-  7.071067811865474617150084668537601828575134277343750000000000e-01,
-  7.400936164611303658134033867099788039922714233398437500000000e-01,
-  7.778619134302061643992942663317080587148666381835937500000000e-01,
-  8.213398158522907666068135768000502139329910278320312500000000e-01,
-  8.717233978105488612087015098950359970331192016601562500000000e-01,
-  9.305794983517888807611484480730723589658737182617187500000000e-01,
-  9.999999999999997779553950749686919152736663818359375000000000e-01,
-  1.082840285100100219395358180918265134096145629882812500000000e+00,
-  1.183100791576249255498964885191526263952255249023437500000000e+00,
-  1.306562964876376353728915091778617352247238159179687500000000e+00,
-  1.461902200081543146126250576344318687915802001953125000000000e+00,
-  1.662754761711521034328598034335300326347351074218750000000000e+00,
-  1.931851652578135070115195048856548964977264404296875000000000e+00,
-  2.310113157672649020213384574162773787975311279296875000000000e+00,
-  2.879385241571815523542454684502445161342620849609375000000000e+00,
-  3.830648787770197127855453800293616950511932373046875000000000e+00,
-  5.736856622834929808618653623852878808975219726562500000000000e+00,
-  1.146279281302667207853573927422985434532165527343750000000000e+01
-};
 
 #define ICOS24(i) (cos24_table[(i)])
 
 #define COS18(i) (cos18_table[(i)])
 #define ICOS36_A(i) (icos72_table[4*(i)+1])
 #define ICOS72_A(i) (icos72_table[2*(i)])
-#else
-/* Non-precalculated version */
-#define ICOS24(i) 0.5 / cos(((i)+1) * (PI/24.0))
 
-#define COS18(i) (cos((i) * PI / 18.0))
-#define ICOS36_A(i) (0.5 / cos((i) * (2*PI/36.0) + (PI/36.0)))
-#define ICOS72_A(i) (0.5 / cos((i) * (2*PI/72.0) + (PI/72.0)))
-#endif
 
-#define POST_TWIDDLE(i,N) (1.0 / (2.0 * cos ((2*i+1) * (PI / (2*N)))))
+/* Static shared computed constants */
+static int mdct_inited = 0;
+static gfloat mdct_swin[4][36];
 
 /* Short (12 point) version of the IMDCT performed
    as 2 x 3-point IMDCT */
@@ -1266,57 +1215,12 @@ imdct_9pt (gfloat invec[9], gfloat outvec[9])
   }
 }
 
-#if 0
-/* Array of windowing values */
-static const gdouble window[4][36] = 
-{
-}
-#endif
-#ifdef OPT_IMDCT
-void
+static void
 inv_mdct (gfloat in[18], gfloat out[36], gint block_type)
 {
   int i, m, p;
 
-  /* Static shared computed constants */
-  static int init = 0;
-  static gfloat swin[4][36];
-
-  const gfloat (*win)[36] = (const gfloat (*)[36])swin;
-
-  if (!init) {
-    /* type 0 */
-    for (i = 0; i < 36; i++)
-      swin[0][i] = sin (PI / 36 * (i + 0.5));
-
-    /* type 1 */
-    for (i = 0; i < 18; i++)
-      swin[1][i] = sin (PI / 36 * (i + 0.5));
-    for (i = 18; i < 24; i++)
-      swin[1][i] = 1.0;
-    for (i = 24; i < 30; i++)
-      swin[1][i] = sin (PI / 12 * (i + 0.5 - 18));
-    for (i = 30; i < 36; i++)
-      swin[1][i] = 0.0;
-
-    /* type 3 */
-    for (i = 0; i < 6; i++)
-      swin[3][i] = 0.0;
-    for (i = 6; i < 12; i++)
-      swin[3][i] = sin (PI / 12 * (i + 0.5 - 6));
-    for (i = 12; i < 18; i++)
-      swin[3][i] = 1.0;
-    for (i = 18; i < 36; i++)
-      swin[3][i] = sin (PI / 36 * (i + 0.5));
-
-    /* type 2 */
-    for (i = 0; i < 12; i++)
-      swin[2][i] = sin (PI / 12 * (i + 0.5));
-    for (i = 12; i < 36; i++)
-      swin[2][i] = 0.0;
-
-    init = 1;
-  }
+  const gfloat (*win)[36] = (const gfloat (*)[36])mdct_swin;
 
   if (block_type == 2) {
     gfloat tmp[12], tin[18], *tmpptr;
@@ -1355,96 +1259,46 @@ inv_mdct (gfloat in[18], gfloat out[36], gint block_type)
     }    
   }
 }
-#else
-void
-inv_mdct (gfloat in[18], gfloat out[36], gint block_type)
+
+static void init_mdct ()
 {
-/*------------------------------------------------------------------*/
-/*                                                                  */
-/*    Function: Calculation of the inverse MDCT                     */
-/*    In the case of short blocks the 3 output vectors are already  */
-/*    overlapped and added in this modul.                           */
-/*                                                                  */
-/*    New layer3                                                    */
-/*                                                                  */
-/*------------------------------------------------------------------*/
+  gint i;
 
-  int i, m, N, p;
-  double tmp[12], sum;
-
-  /* Static shared computed constants */
-  static int init = 0;
-  static gfloat swin[4][36];
-  static gfloat sCOS[4 * 36];
-
-  const gfloat (*win)[36] = (const gfloat (*)[36])swin;
-  const gfloat *COS = sCOS;
-
-  if (!init) {
-    /* type 0 */
-    for (i = 0; i < 36; i++)
-      swin[0][i] = sin (PI / 36 * (i + 0.5));
-
-    /* type 1 */
-    for (i = 0; i < 18; i++)
-      swin[1][i] = sin (PI / 36 * (i + 0.5));
-    for (i = 18; i < 24; i++)
-      swin[1][i] = 1.0;
-    for (i = 24; i < 30; i++)
-      swin[1][i] = sin (PI / 12 * (i + 0.5 - 18));
-    for (i = 30; i < 36; i++)
-      swin[1][i] = 0.0;
-
-    /* type 3 */
-    for (i = 0; i < 6; i++)
-      swin[3][i] = 0.0;
-    for (i = 6; i < 12; i++)
-      swin[3][i] = sin (PI / 12 * (i + 0.5 - 6));
-    for (i = 12; i < 18; i++)
-      swin[3][i] = 1.0;
-    for (i = 18; i < 36; i++)
-      swin[3][i] = sin (PI / 36 * (i + 0.5));
-
-    /* type 2 */
-    for (i = 0; i < 12; i++)
-      swin[2][i] = sin (PI / 12 * (i + 0.5));
-    for (i = 12; i < 36; i++)
-      swin[2][i] = 0.0;
-
-    for (i = 0; i < 4 * 36; i++)
-      sCOS[i] = cos (PI / (2 * 36) * i);
-
-    init = 1;
-  }
+  if (G_UNLIKELY (mdct_inited))
+    return;
 
+  /* type 0 */
   for (i = 0; i < 36; i++)
-    out[i] = 0;
+    mdct_swin[0][i] = sin (PI / 36 * (i + 0.5));
 
-  if (block_type == 2) {
-    N = 12;
-    for (i = 0; i < 3; i++) {
-      for (p = 0; p < N; p++) {
-        sum = 0.0;
-        for (m = 0; m < N / 2; m++)
-          sum +=
-              in[i + 3 * m] * cos (PI / (2 * N) * (2 * p + 1 + N / 2) * (2 * m +
-                  1));
-        tmp[p] = sum * win[block_type][p];
-      }
-      for (p = 0; p < N; p++)
-        out[6 * i + p + 6] += tmp[p];
-    }
-  } else {
-    N = 36;
-    for (p = 0; p < N; p++) {
-      sum = 0.0;
-      for (m = 0; m < N / 2; m++)
-        sum += in[m] * COS[((2 * p + 1 + N / 2) * (2 * m + 1)) % (4 * 36)];
-      out[p] = sum * win[block_type][p];
-    }
-  }
+  /* type 1 */
+  for (i = 0; i < 18; i++)
+    mdct_swin[1][i] = sin (PI / 36 * (i + 0.5));
+  for (i = 18; i < 24; i++)
+    mdct_swin[1][i] = 1.0;
+  for (i = 24; i < 30; i++)
+    mdct_swin[1][i] = sin (PI / 12 * (i + 0.5 - 18));
+  for (i = 30; i < 36; i++)
+    mdct_swin[1][i] = 0.0;
+
+  /* type 3 */
+  for (i = 0; i < 6; i++)
+    mdct_swin[3][i] = 0.0;
+  for (i = 6; i < 12; i++)
+    mdct_swin[3][i] = sin (PI / 12 * (i + 0.5 - 6));
+  for (i = 12; i < 18; i++)
+    mdct_swin[3][i] = 1.0;
+  for (i = 18; i < 36; i++)
+    mdct_swin[3][i] = sin (PI / 36 * (i + 0.5));
+
+  /* type 2 */
+  for (i = 0; i < 12; i++)
+    mdct_swin[2][i] = sin (PI / 12 * (i + 0.5));
+  for (i = 12; i < 36; i++)
+    mdct_swin[2][i] = 0.0;
+
+  mdct_inited = 1;
 }
-#endif
 
 void
 init_hybrid (mp3cimpl_info *c_impl)
@@ -1465,7 +1319,7 @@ init_hybrid (mp3cimpl_info *c_impl)
  *   gr_info_t *gr_info
  *   frame_params *fr_ps
  */
-void
+static void
 III_hybrid (gfloat fsIn[SSLIMIT], gfloat tsOut[SSLIMIT], int sb, int ch,
     gr_info_t *gr_info, mp3tl *tl)
 {
@@ -1484,14 +1338,29 @@ III_hybrid (gfloat fsIn[SSLIMIT], gfloat tsOut[SSLIMIT], int sb, int ch,
   }
 }
 
+/* Invert the odd frequencies for odd subbands in preparation for polyphase
+ * filtering */
+static void
+III_frequency_inversion (gfloat hybridOut[SBLIMIT][SSLIMIT], 
+    mp3tl *tl __attribute__ ((unused)))
+{
+  guint ss, sb;
+
+  for (ss = 1; ss < 18; ss += 2) {
+    for (sb = 1; sb < SBLIMIT; sb += 2) {
+      hybridOut[sb][ss] = -hybridOut[sb][ss];
+    }
+  }
+}
+
 Mp3TlRetcode
 decode_mp3 (mp3tl *tl)
 {
   III_scalefac_t III_scalefac;
   III_side_info_t III_side_info;
   huffdec_bitbuf *bb;
-  int nSlots;
-  int gr, ch, ss, sb, main_data_end, flush_main;
+  guint nSlots;
+  guint gr, ch, sb;
   int bytes_to_discard;
   fr_header *hdr;
   
@@ -1508,31 +1377,22 @@ decode_mp3 (mp3tl *tl)
     h_putbyte (bb, bs_getbits (tl->bs, 8));
 
   /* Check the position in the huffman decoder after the last 
-    * frame. Drop any remaining bits to start the new frame on a slot
-    * boundary */
-  /* End position of the (previous?) frame: */
-  main_data_end = h_sstell (bb) / hdr->bits_per_slot; 
-  flush_main = (h_sstell (bb) % hdr->bits_per_slot);
-  if (flush_main != 0) {
-    h_getbits (bb, (gint)(hdr->bits_per_slot) - flush_main);
-    main_data_end++;
-  }
-  bytes_to_discard = tl->c_impl.frame_start - main_data_end
-      - III_side_info.main_data_begin;
-  if (main_data_end > 4096) {
-    // g_print ("Rewinding huffman decoder by 4096 bytes, why?\n");
-    tl->c_impl.frame_start -= 4096;
-    h_rewindNbytes (bb, 4096);
-  }
+   * frame. Drop any remaining bits to start the new frame on a slot
+   * boundary */
+  h_byte_align (bb);
+  bytes_to_discard = 
+      h_bytes_avail (bb) - (III_side_info.main_data_begin + hdr->main_slots);
 
-  tl->c_impl.frame_start += hdr->main_slots;
-  if (bytes_to_discard < 0) {
 #if 0
-    g_warning
-        ("Not enough main data to decode frame %" G_GUINT64_FORMAT
-        ".  Frame discarded.\n", tl->frame_num - 1);
+  g_print ("Mode %d mode_ext %d\n", hdr->mode, hdr->mode_ext);
+  g_print ("Frame: main_data %d bytes. need %d prev bytes. Avail %d (of %d)\n", 
+      hdr->main_slots, III_side_info.main_data_begin, h_bytes_avail (bb),
+      III_side_info.main_data_begin + hdr->main_slots);
 #endif
-    GST_DEBUG ("Not enough main data available to decode frame");
+
+  if (bytes_to_discard < 0) {
+    /* Usually happens after a seek. We can't decode this frame */
+    GST_LOG ("Not enough main data available to decode frame");
     return MP3TL_ERR_BAD_FRAME;
   }
   for (; bytes_to_discard > 0; bytes_to_discard--)
@@ -1542,7 +1402,7 @@ decode_mp3 (mp3tl *tl)
     gfloat lr[2][SBLIMIT][SSLIMIT], ro[2][SBLIMIT][SSLIMIT];
 
     for (ch = 0; ch < hdr->channels; ch++) {
-      glong is[SBLIMIT][SSLIMIT];        /* Quantized samples. */
+      gint is[SBLIMIT][SSLIMIT];        /* Quantized samples. */
       int part2_start;
 
       part2_start = h_sstell (bb);
@@ -1566,7 +1426,19 @@ decode_mp3 (mp3tl *tl)
 
       III_dequantize_sample (is, ro[ch], &III_scalefac,
           &(III_side_info.gr[gr][ch]), ch, &tl->fr_ps);
+
+#if 0
+      int i;
+      for (sb = 0; sb < SBLIMIT; sb++) {
+        g_print ("SB %02d: ", sb);
+        for (i = 0; i < SSLIMIT; i++) {
+          g_print ("%06f ", ro[ch][sb][i]);
+        }
+        g_print ("\n");
+      }
+#endif
     }
+
     III_stereo (ro, lr, &III_scalefac,
         &(III_side_info.gr[gr][0]), &tl->fr_ps);
 
@@ -1574,13 +1446,23 @@ decode_mp3 (mp3tl *tl)
       gfloat re[SBLIMIT][SSLIMIT];
       gfloat hybridIn[SBLIMIT][SSLIMIT];    /* Hybrid filter input */
       gfloat hybridOut[SBLIMIT][SSLIMIT];   /* Hybrid filter out */
-      gfloat polyPhaseIn[SBLIMIT];  /* PolyPhase Input. */
       gr_info_t *gi = &(III_side_info.gr[gr][ch]);
       
       III_reorder (lr[ch], re, gi, &tl->fr_ps);
       
       /* Antialias butterflies. */
-      III_antialias (re, hybridIn, gi, &tl->fr_ps);
+      III_antialias (re, hybridIn, gi);
+#if 0
+      int i;
+      g_print ("HybridIn\n");
+      for (sb = 0; sb < SBLIMIT; sb++) {
+        g_print ("SB %02d: ", sb);
+        for (i = 0; i < SSLIMIT; i++) {
+          g_print ("%06f ", hybridIn[sb][i]);
+        }
+        g_print ("\n");
+      }
+#endif
 
       for (sb = 0; sb < SBLIMIT; sb++) {
         /* Hybrid synthesis. */
@@ -1588,87 +1470,52 @@ decode_mp3 (mp3tl *tl)
       }
 
       /* Frequency inversion for polyphase. Invert odd numbered indices */
-      for (ss = 1; ss < 18; ss += 2)
-        for (sb = 1; sb < SBLIMIT; sb += 2) {
-          hybridOut[sb][ss] = -hybridOut[sb][ss];
+      III_frequency_inversion (hybridOut, tl);
+
+#if 0
+      g_print ("HybridOut\n");
+      for (sb = 0; sb < SBLIMIT; sb++) {
+        g_print ("SB %02d: ", sb);
+        for (i = 0; i < SSLIMIT; i++) {
+          g_print ("%06f ", hybridOut[sb][i]);
         }
+        g_print ("\n");
+      }
+#endif
 
       /* Polyphase synthesis */
-      for (ss = 0; ss < 18; ss++) {
-        for (sb = 0; sb < SBLIMIT; sb++)
-          polyPhaseIn[sb] = hybridOut[sb][ss];
-          
-          mp3_SubBandSynthesis (tl, &tl->fr_ps, polyPhaseIn, ch,
-            &(tl->pcm_sample[ch][ss][0]));
+      III_subband_synthesis (tl, &tl->fr_ps, hybridOut, ch, tl->pcm_sample[ch]);
+#if 0
+      if (ch == 0) {
+        g_print ("synth\n");
+
+        for (i = 0; i < SSLIMIT; i++) {
+          g_print ("SS %02d: ", i);
+          for (sb = 0; sb < SBLIMIT; sb++) {
+            g_print ("%04d ", tl->pcm_sample[ch][sb][i]);
+        }
+        g_print ("\n");
       }
+}
+#endif
+
     }
     /* Output PCM sample points for one granule. */
     out_fifo (tl->pcm_sample, 18, &tl->fr_ps, tl->sample_buf, 
         &tl->sample_w, SAMPLE_BUF_SIZE);
   }
+#if 0
+  g_print ("After: %d bytes left\n", h_bytes_avail (bb));
+#endif
   
   return MP3TL_ERR_OK;
 }
 
-void
-mp3_SubBandSynthesis (mp3tl *tl, frame_params *fr_ps,
-    float *bandPtr, gint channel, short *samples)
-{
-  int i, j, k;
-  float *bufOffsetPtr, sum;
-  gint    *bufOffset;
-  float *synbuf = fr_ps->synbuf[channel];
-
-  bufOffset = fr_ps->bufOffset;
-
-  bufOffset[channel] = (bufOffset[channel] - 64) & 0x3ff;
-  /* g_print ("bufOffset[%d] = %d\n", channel, bufOffset[channel]); */
-
-  bufOffsetPtr = synbuf + bufOffset[channel];
-
-  for (i = 0; i < 64; i++) {
-    float* f_row = fr_ps->filter[i];
-    sum = 0;
-    for (k = 0; k < 32; k++)
-      sum += bandPtr[k] * f_row[k];
-    bufOffsetPtr[i] = sum;
-  }
-
-  /*  S(i,j) = D(j+32i) * U(j+32i+((i+1)>>1)*64)  */
-  /*  samples(i,j) = MWindow(j+32i) * bufPtr(j+32i+((i+1)>>1)*64)  */
-  for (j = 0; j < 32; j++) {
-    sum = 0;
-    sum += dewindow[j] * synbuf[(j + bufOffset[channel]) & 0x3ff];
-    for (i = 64; i < 512; i += 64) {
-      k = j + i;
-      sum += dewindow[k] * synbuf[(k + i + bufOffset[channel]) & 0x3ff];
-      sum += dewindow[k - 32] * synbuf[(k - 32 + i + bufOffset[channel]) & 0x3ff];
-    }
-
-    /* Casting truncates towards zero for both positive and negative numbers,
-       the result is cross-over distortion,  1995-07-12 shn */
-    if (sum > 0) {
-      if (sum < 1.0) {
-        samples[j] = (short) (sum * SCALE + 0.5);
-      } else {
-        samples[j] = (short) (SCALE - 1);
-      }
-    } else {
-      if (sum > -1.0) {
-        samples[j] = (short) (sum * SCALE - 0.5);
-      } else {
-        samples[j] = (short) (-SCALE);
-      }
-    }
-  }
-}
-
 gboolean mp3_init (mp3tl *tl)
 {
-  tl->c_impl.frame_start = 0;
-
   h_init (&tl->c_impl.bb);
   init_hybrid (&tl->c_impl);  
+  init_mdct ();
   return TRUE;
 }
 
@@ -1676,10 +1523,9 @@ void mp3_flush (mp3tl *tl)
 {
   h_flush (&tl->c_impl.bb);
   init_hybrid (&tl->c_impl);  
-  tl->c_impl.frame_start = 0;
 }
 
-void mp3_close (mp3tl *tl)
+void mp3_close (mp3tl *tl __attribute__ ((unused)))
 {
 }
 
